@@ -12,10 +12,8 @@ SPDX-License-Identifier: MIT
 #ifndef IGA_H
 #define IGA_H
 
-#include "iga_types_swsb.hpp"
-
-#include <stdint.h>
 #include <stddef.h>
+#include <stdint.h>
 #ifdef _WIN32
 #include <crtdefs.h>
 #endif
@@ -30,37 +28,37 @@ SPDX-License-Identifier: MIT
 #define IGA_API
 #endif
 #else
-    #if __GNUC__ >= 4
-        #define IGA_API __attribute__ ((visibility ("default")))
-    #else
+#if __GNUC__ >= 4
+#define IGA_API __attribute__((visibility("default")))
+#else
 #define IGA_API
-    #endif
+#endif
 #endif
 
 #ifdef __cplusplus
-extern "C"  {
+extern "C" {
 #endif
 
 /* opaque pointer to context type */
 typedef void *iga_context_t;
 
 typedef enum {
-    IGA_SUCCESS        = 0, /* the operation completed successfully
-                             * (iga_disassemble and iga_assemble may
-                             * still have logged warnings) */
-    IGA_ERROR          = 1, /* general error */
-    IGA_INVALID_ARG    = 2, /* something wrong with an arugment */
-    IGA_OUT_OF_MEM     = 3, /* failed to allocate */
-    IGA_DECODE_ERROR   = 4, /* error during decode phase */
-    IGA_ENCODE_ERROR   = 5, /* error during the encode phase */
-    IGA_PARSE_ERROR    = 6, /* error duing the parse phase (syntax) */
-    IGA_VERSION_ERROR  = 7, /* this header file is newer than the binary
-                             * called */
-    IGA_INVALID_OBJECT = 8, /* attempt to use an destroyed object
-                             * (e.g. iga_context_t) */
-    IGA_INVALID_STATE  = 9, /* e.g. call to iga_get_errors before disassembly*/
-    IGA_UNSUPPORTED_PLATFORM = 10, /* platform not supported with this binary */
-    IGA_DIFF_FAILURE = 11 /* used by -Xifs for diffing instructions */
+  IGA_SUCCESS = 0,        /* the operation completed successfully
+                           * (iga_disassemble and iga_assemble may
+                           * still have logged warnings) */
+  IGA_ERROR = 1,          /* general error */
+  IGA_INVALID_ARG = 2,    /* something wrong with an arugment */
+  IGA_OUT_OF_MEM = 3,     /* failed to allocate */
+  IGA_DECODE_ERROR = 4,   /* error during decode phase */
+  IGA_ENCODE_ERROR = 5,   /* error during the encode phase */
+  IGA_PARSE_ERROR = 6,    /* error duing the parse phase (syntax) */
+  IGA_VERSION_ERROR = 7,  /* this header file is newer than the binary
+                           * called */
+  IGA_INVALID_OBJECT = 8, /* attempt to use an destroyed object
+                           * (e.g. iga_context_t) */
+  IGA_INVALID_STATE = 9,  /* e.g. call to iga_get_errors before disassembly*/
+  IGA_UNSUPPORTED_PLATFORM = 10, /* platform not supported with this binary */
+  IGA_DIFF_FAILURE = 11          /* used by -Xifs for diffing instructions */
 } iga_status_t;
 
 /*
@@ -70,48 +68,51 @@ typedef enum {
  */
 IGA_API const char *iga_status_to_string(iga_status_t st);
 
-
 /*
-* Returns a NUL-terminated version string corresponding to the version of
-* IGA.
-*/
+ * Returns a NUL-terminated version string corresponding to the version of IGA.
+ */
 IGA_API const char *iga_version_string();
 
-
 /* The encoding for GEN version enumerates follows this pattern */
-#define GEN_VER(MAJ,MIN) (((MAJ)<<16)|(MIN))
+#define GEN_VER(MAJ, MIN) (((MAJ) << 16) | (MIN))
 
 /* The encoding for XE version enumerates follows this pattern
-* All XE_VER() must be larger than GEN_VER().  XE_VER(0,*) is illegal. TGL is XE_VER(1,0)
-*/
-#define XE_VER(MAJ,MIN) (((MAJ)<<24)|(MIN))
-
+ * All XE_VER() must be larger than GEN_VER().  XE_VER(0,*) is illegal. TGL is
+ * XE_VER(1,0)
+ */
+#define XE_VER(MAJ, MIN) (((MAJ) << 24) | (MIN))
 
 /* Represents the specific version of Gen being assembled or disassembled */
 typedef enum {
-    IGA_GEN_INVALID = 0
-  , IGA_GEN7      = GEN_VER(7,0)
-  , IGA_GEN7p5    = GEN_VER(7,5)
-  , IGA_GEN8      = GEN_VER(8,0)
-  , IGA_GEN8lp    = GEN_VER(8,1)
-  , IGA_GEN9      = GEN_VER(9,0)
-  , IGA_GEN9lp    = GEN_VER(9,1)
-  , IGA_GEN9p5    = GEN_VER(9,5)
-  , IGA_GEN10     = GEN_VER(10,0)
-  , IGA_GEN11     = GEN_VER(11,0)
+  IGA_GEN_INVALID = 0,
+  IGA_GEN7 = GEN_VER(7, 0),
+  IGA_GEN7p5 = GEN_VER(7, 5),
+  IGA_GEN8 = GEN_VER(8, 0),
+  IGA_GEN8lp = GEN_VER(8, 1),
+  IGA_GEN9 = GEN_VER(9, 0),
+  IGA_GEN9lp = GEN_VER(9, 1),
+  IGA_GEN9p5 = GEN_VER(9, 5),
+  IGA_GEN10 = GEN_VER(10, 0),
+  IGA_GEN11 = GEN_VER(11, 0),
+  //
   // XE versions
-  , IGA_XE        = XE_VER(1, 0) // TGL
-  , IGA_XE_HP     = XE_VER(1, 1)
-  , IGA_XE_HPG    = XE_VER(1, 2)
-  , IGA_XE_HPC    = XE_VER(1, 4)
+  IGA_XE = XE_VER(1, 0), // TGL
+  IGA_XE_HP = XE_VER(1, 1),
+  IGA_XE_HPG = XE_VER(1, 2),
+  IGA_XE_HPC = XE_VER(1, 4)
+  ,
+  IGA_XE2 = XE_VER(2, 0)
+  ,
+  IGA_XE3 = XE_VER(3, 0)
+  , IGA_XE3P_XPC = XE_VER(3, 3)
 
-  // TO BE DEPRECATED
+  // DEPRECATED
   // Preserve the old values to maintain the binary compatibility
   // The value should not be used anymore
-  , IGA_GEN12p1  = GEN_VER(12,1)  // IGA_XE
+  ,
+  IGA_GEN12p1 = GEN_VER(12, 1) // IGA_XE
 } iga_gen_t;
 
-
 /*****************************************************************************/
 /*                     Platform Query Functions                              */
 /*****************************************************************************/
@@ -133,10 +134,10 @@ typedef enum {
  *   IGA_INVALID_ARGUMENT  if gens_length_bytes != 0 && gens == nullptr
  *   IGA_SUCCESS           otherwise
  */
-IGA_API iga_status_t iga_platforms_list(
-  size_t gens_length_bytes,
-  iga_gen_t *gens,
-  size_t *gens_length_bytes_required);
+IGA_API iga_status_t iga_platforms_list(size_t gens_length_bytes,
+                                        iga_gen_t *gens,
+                                        size_t *gens_length_bytes_required);
+
 /*
  * Returns the platform suffix name of a given platform.
  *
@@ -151,9 +152,9 @@ IGA_API iga_status_t iga_platforms_list(
  *                         supported by this platform
  *   IGA_SUCCESS           otherwise
  */
-IGA_API iga_status_t iga_platform_symbol_suffix(
-    iga_gen_t gen,
-    const char **suffix);
+IGA_API iga_status_t iga_platform_symbol_suffix(iga_gen_t gen,
+                                                const char **suffix);
+
 /*
  * Returns the names for a given platform.  E.g. IGA_GEN9 returns "skl".
  *
@@ -170,35 +171,28 @@ IGA_API iga_status_t iga_platform_symbol_suffix(
  *   IGA_INVALID_ARGUMENT  if names_bytes != 0 && names == nullptr
  *   IGA_SUCCESS           otherwise
  */
-IGA_API iga_status_t iga_platform_names(
-    iga_gen_t gen,
-    size_t names_bytes,
-    const char **names,
-    size_t *names_bytes_needed);
-
-
+IGA_API iga_status_t iga_platform_names(iga_gen_t gen, size_t names_bytes,
+                                        const char **names,
+                                        size_t *names_bytes_needed);
 
 /*****************************************************************************/
 /*                  Context Creation Functions                               */
 /*****************************************************************************/
 
 /*
- * Context options
+ * Context creation options
  */
 typedef struct {
-    size_t        cb;   /* set to sizeof(iga_context_options_t) */
-    iga_gen_t     gen;
+  size_t cb; /* set to sizeof(iga_context_options_t) */
+  iga_gen_t gen;
 } iga_context_options_t;
-/* this is an ugly wart, cb should have been uint32_t *
- * TODO: change this in IGA 2.0 and break binary compatibility there*/
 
 /*
  * This macro initializes options to the context.
  *  iga_gen_t   P    the gen platform to use
  */
-#define IGA_CONTEXT_OPTIONS_INIT(P) \
-    {sizeof(iga_context_options_t), (P)}
-
+#define IGA_CONTEXT_OPTIONS_INIT(P)                                            \
+  { sizeof(iga_context_options_t), (P) }
 
 /*
  * Creates a context, which is needed to assemble and disassemble kernels.
@@ -216,14 +210,11 @@ typedef struct {
  *  IGA_OUT_OF_MEM        upon internal allocation failure
  *  IGA_INVALID_PLATFORM  if the platform passed is unsupported
  */
-IGA_API iga_status_t  iga_context_create(
-    const iga_context_options_t *opts,
-    iga_context_t *ctx);
+IGA_API iga_status_t iga_context_create(const iga_context_options_t *opts,
+                                        iga_context_t *ctx);
 /* deprecated: covers to iga_context_create */
-IGA_API iga_status_t  iga_create_context(
-    const iga_context_options_t *opts,
-    iga_context_t *ctx);
-
+IGA_API iga_status_t iga_create_context(const iga_context_options_t *opts,
+                                        iga_context_t *ctx);
 
 /*
  * Releases a context previously created via 'iga_context_create'.
@@ -233,10 +224,9 @@ IGA_API iga_status_t  iga_create_context(
  *  IGA_INVALID_ARG     if an argument is NULL
  *  IGA_INVALID_OBJECT  if ctx has already been destroyed
  */
-IGA_API iga_status_t  iga_context_release(iga_context_t ctx);
+IGA_API iga_status_t iga_context_release(iga_context_t ctx);
 /* deprecated: covers to iga_context_release */
-IGA_API iga_status_t  iga_release_context(iga_context_t ctx);
-
+IGA_API iga_status_t iga_release_context(iga_context_t ctx);
 
 /*****************************************************************************/
 /*                  Assembly Functions                                       */
@@ -246,33 +236,35 @@ IGA_API iga_status_t  iga_release_context(iga_context_t ctx);
  * This structure contains options to the 'iga_assemble' call.
  */
 typedef struct {
-    uint32_t     cb;                 /* sizeof(iga_context_options_t)    */
-    uint32_t     enabled_warnings;   /* bitset of IGA_WARNINGS*          */
-    uint32_t     encoder_opts;       /* bitset of IGA_ENCODER_OPT*       */
-    uint32_t     syntax_opts;        /* bitset of IGA_SYNTAX_OPT*        */
-     /* the following must be 0 or a warning is raised */
-    uint32_t     _reserved0;         /* use IGA_ENCODER_OPT_ERRONCOMPACT */
-    uint32_t     _reserved1;         /* use IGA_ENCODER_OPT_AUTODEP      */
-    /*
-     * ... future fields (ensure total size is a multiple of 8;
-     * add "reserved" if needed) ...
-     * do not reuse _reserved0 and _reserved1 since those were
-     * historically something else
-     */
+  uint32_t cb;               /* sizeof(iga_context_options_t)    */
+  uint32_t enabled_warnings; /* bitset of IGA_WARNINGS*          */
+  uint32_t encoder_opts;     /* bitset of IGA_ENCODER_OPT*       */
+  uint32_t syntax_opts;      /* bitset of IGA_SYNTAX_OPT*        */
+  /* the following must be 0 or a warning is raised */
+  uint32_t _reserved0; /* use IGA_ENCODER_OPT_ERRONCOMPACT */
+  uint32_t _reserved1; /* use IGA_ENCODER_OPT_AUTODEP      */
+  /*
+   * ... future fields (ensure total size is a multiple of 8;
+   * add "reserved" if needed) ...
+   * do not reuse _reserved0 and _reserved1 since those were
+   * historically something else
+   */
 
-    // number of sbid used for auto dependency setting. This value is effective
-    // only when IGA_ENCODER_OPT_AUTO_DEPENDENCIES is given
-    uint32_t sbid_count;
-    // force the swsb_encode_mode. If not given, the encode mode will be
-    // derived from platform
-    iga::SWSB_ENCODE_MODE swsb_encode_mode;
+  /* number of sbid used for auto dependency setting. This value is effective
+   * only when IGA_ENCODER_OPT_AUTO_DEPENDENCIES is given */
+  uint32_t sbid_count;
+
+  /*
+   * Force the swsb_encode_mode.  If not given (SWSBInvalidMode),
+   * then encode mode will be derived from platform.
+   */
+  uint32_t swsb_encode_mode; /* use iga::SWSB_ENCODE_MODE */
 } iga_assemble_options_t;
 
 /* detects screwups where someone adds a field and the compiler pads
  * the structure out implicitly */
-static_assert(sizeof(iga_assemble_options_t) == 8*4,
-    "wrong size for iga_assemble_options");
-
+static_assert(sizeof(iga_assemble_options_t) == 8 * 4,
+              "wrong size for iga_assemble_options");
 
 /*
  * encoding options
@@ -284,59 +276,55 @@ static_assert(sizeof(iga_assemble_options_t) == 8*4,
  *    op (...) ... { }           // try and compact iff IGA_ENCOPTS_AUTOCOMPACT
  *    op (...) ... {Uncompacted} // will not attempt to compact
  */
-#define IGA_ENCODER_OPT_AUTO_COMPACT            0x00000001u
+#define IGA_ENCODER_OPT_AUTO_COMPACT 0x00000001u
 /* auto set instruction dependencies */
-#define IGA_ENCODER_OPT_AUTO_DEPENDENCIES       0x00000002u
+#define IGA_ENCODER_OPT_AUTO_DEPENDENCIES 0x00000002u
 /* treat failure to compact an instruction with a {Compacted} annotation
-* as a hard error rather than just raising a warning */
-#define IGA_ENCODER_OPT_ERROR_ON_COMPACT_FAIL   0x00000004u
+ * as a hard error rather than just raising a warning */
+#define IGA_ENCODER_OPT_ERROR_ON_COMPACT_FAIL 0x00000004u
 /* enable experimental native encoder */
-#define IGA_ENCODER_OPT_USE_NATIVE              0x00000008u
-/* forcely NoCompact to all instructions even if {Compacted} is set on the instruction
-   This option will overried IGA_ENCODER_OPT_AUTO_COMPACT */
-#define IGA_ENCODER_OPT_FORCE_NO_COMPACT        0x00000010u
-
+#define IGA_ENCODER_OPT_USE_NATIVE 0x00000008u
+/* forcely NoCompact to all instructions even if {Compacted} is set on the
+   instruction This option will overried IGA_ENCODER_OPT_AUTO_COMPACT */
+#define IGA_ENCODER_OPT_FORCE_NO_COMPACT 0x00000010u
+/* auto set instruction dependencies with sbid counter. This option takes
+   effect only when IGA_ENCODER_OPT_AUTO_DEPENDENCIES is set */
+#define IGA_ENCODER_OPT_AUTO_SBID_COUNTER 0x00000020u
 /*
  * options for the parsing phase
  */
 /* nominal support for certain IsaAsm directives and syntax */
-#define IGA_SYNTAX_OPT_LEGACY_SYNTAX   0x00000001u
+#define IGA_SYNTAX_OPT_LEGACY_SYNTAX 0x00000001u
 /* enables syntax extensions */
-#define IGA_SYNTAX_OPT_EXTENSIONS      0x00000002u
-
+#define IGA_SYNTAX_OPT_EXTENSIONS 0x00000002u
 
 /*
  * extra assemble warnings (things to check)
  * (for iga_assemble_options_t::enabled_warnings)
  */
 /* individual warnings */
-#define IGA_WARNINGS_REGIONS    0x00000001u  /* -Wregions */
-#define IGA_WARNINGS_TYPES      0x00000002u  /* -Wtypes */
-#define IGA_WARNINGS_SCHED      0x00000004u  /* -Wscheduling */
-#define IGA_WARNINGS_NORMFORM   0x00000008u  /* -Wnormal-form */
+#define IGA_WARNINGS_REGIONS 0x00000001u  /* -Wregions */
+#define IGA_WARNINGS_TYPES 0x00000002u    /* -Wtypes */
+#define IGA_WARNINGS_SCHED 0x00000004u    /* -Wscheduling */
+#define IGA_WARNINGS_NORMFORM 0x00000008u /* -Wnormal-form */
 /* useful predefined sets */
-#define IGA_WARNINGS_NONE       0x00000000u  /* -Wnone */
-#define IGA_WARNINGS_ALL \
-    (IGA_WARNINGS_REGIONS|\
-    IGA_WARNINGS_TYPES|\
-    IGA_WARNINGS_SCHED|\
-    IGA_WARNINGS_NORMFORM)  /* -Wall */
-#define IGA_WARNINGS_DEFAULT \
-    (IGA_WARNINGS_REGIONS|\
-    IGA_WARNINGS_SCHED) /* -Wdefault */
+#define IGA_WARNINGS_NONE 0x00000000u /* -Wnone */
+#define IGA_WARNINGS_ALL                                                       \
+  (IGA_WARNINGS_REGIONS | IGA_WARNINGS_TYPES | IGA_WARNINGS_SCHED |            \
+   IGA_WARNINGS_NORMFORM) /* -Wall */
+#define IGA_WARNINGS_DEFAULT                                                   \
+  (IGA_WARNINGS_REGIONS | IGA_WARNINGS_SCHED) /* -Wdefault */
 
 /* helpful initializers for assembly options with good defaults */
-#define IGA_ASSEMBLE_OPTIONS_INIT() \
-    { \
-      sizeof(iga_assemble_options_t), \
-      IGA_WARNINGS_DEFAULT, \
-      (IGA_ENCODER_OPT_ERROR_ON_COMPACT_FAIL), \
-      0, /* syntax_opts = NONE */ \
-      0, /* reserved */  \
-      0, /* reserved */  \
-      16, /* sbid_count */ \
-      iga::SWSB_ENCODE_MODE::SWSBInvalidMode \
-    }
+#define IGA_ASSEMBLE_OPTIONS_INIT()                                            \
+  {                                                                            \
+    sizeof(iga_assemble_options_t), IGA_WARNINGS_DEFAULT,                      \
+        (IGA_ENCODER_OPT_ERROR_ON_COMPACT_FAIL), 0, /* syntax_opts = NONE */   \
+        0,                                          /* reserved */             \
+        0,                                          /* reserved */             \
+        16,                                         /* sbid_count */           \
+        0,                                          /* SWSBInvalidMode */      \
+  }
 
 /*
  * Assembles some text into bits.
@@ -362,20 +350,15 @@ static_assert(sizeof(iga_assemble_options_t) == 8*4,
  *                      (use iga_get_errors for more info)
  *  IGA_ERROR           upon some other error
  */
-IGA_API iga_status_t  iga_context_assemble(
-    iga_context_t ctx,
-    const iga_assemble_options_t *opts,
-    const char *kernel_text,
-    void **output,
-    uint32_t *output_size);
+IGA_API iga_status_t iga_context_assemble(iga_context_t ctx,
+                                          const iga_assemble_options_t *opts,
+                                          const char *kernel_text,
+                                          void **output, uint32_t *output_size);
 /* deprecated API covers to iga_contex* */
-IGA_API iga_status_t  iga_assemble(
-    iga_context_t ctx,
-    const iga_assemble_options_t *opts,
-    const char *kernel_text,
-    void **output,
-    uint32_t *output_size);
-
+IGA_API iga_status_t iga_assemble(iga_context_t ctx,
+                                  const iga_assemble_options_t *opts,
+                                  const char *kernel_text, void **output,
+                                  uint32_t *output_size);
 
 /*****************************************************************************/
 /*                  Disassembly Functions                                    */
@@ -385,78 +368,81 @@ IGA_API iga_status_t  iga_assemble(
  * This structure contains options to the 'iga_disassemble' call.
  */
 typedef struct {
-    uint32_t     cb;   /* set to sizeof(iga_context_options_t) */
-    uint32_t     formatting_opts; /* options in formatting the syntax */
-    uint32_t     _reserved0; /* use formatting_opts; set this to 0! */
-    uint32_t     _reserved1; /* use formatting_opts; set this to 0! */
-    uint32_t     decoder_opts; /* opts for the decoding phase */
-    /* ... future fields (ensure total size is a multiple of 8;
-     * add "reserved" if needed) ... */
+  uint32_t cb;              /* set to sizeof(iga_context_options_t) */
+  uint32_t formatting_opts; /* options in formatting the syntax */
+  uint32_t _reserved0;      /* use formatting_opts; set this to 0! */
+  uint32_t _reserved1;      /* use formatting_opts; set this to 0! */
+  uint32_t decoder_opts;    /* opts for the decoding phase */
+  uint32_t base_pc_offset;  /* base pc offset to add to pc in disassembly string
+                               output*/
+  /* ... future fields (ensure total size is a multiple of 8;
+   * add "reserved" if needed) ... */
 } iga_disassemble_options_t;
 
-static_assert(sizeof(iga_disassemble_options_t) == 5*4,
-    "wrong size for iga_disassemble_options_t");
+static_assert(sizeof(iga_disassemble_options_t) == 6 * 4,
+              "wrong size for iga_disassemble_options_t");
 
 /* A default value for iga_disassemble_options_t */
-#define IGA_DISASSEMBLE_OPTIONS_INIT() \
-    {sizeof(iga_disassemble_options_t), \
-     IGA_FORMATTING_OPTS_DEFAULT, \
-     0, /* _reserved0 */ \
-     0,  /* _reserved1 */ \
-     IGA_DECODING_OPTS_DEFAULT, /* decoder_opts */ }
+#define IGA_DISASSEMBLE_OPTIONS_INIT()                                         \
+  {                                                                            \
+    sizeof(iga_disassemble_options_t), IGA_FORMATTING_OPTS_DEFAULT,            \
+        0,                         /* _reserved0 */                            \
+        0,                         /* _reserved1 */                            \
+        IGA_DECODING_OPTS_DEFAULT, /* decoder_opts */                          \
+  }
 
 /* A default value for iga_disassemble_options_t that enables numeric labels */
-#define IGA_DISASSEMBLE_OPTIONS_INIT_NUMERIC_LABELS() \
-    {sizeof(iga_disassemble_options_t), \
-     IGA_FORMATTING_OPTS_DEFAULT|IGA_FORMATTING_OPT_NUMERIC_LABELS, \
-     0, /* _reserved0 */ \
-     0, /* _reserved1 */ \
-     IGA_DECODING_OPTS_DEFAULT, /* decoder_opts */ }
+#define IGA_DISASSEMBLE_OPTIONS_INIT_NUMERIC_LABELS()                          \
+  {                                                                            \
+    sizeof(iga_disassemble_options_t),                                         \
+        IGA_FORMATTING_OPTS_DEFAULT | IGA_FORMATTING_OPT_NUMERIC_LABELS,       \
+        0,                         /* _reserved0 */                            \
+        0,                         /* _reserved1 */                            \
+        IGA_DECODING_OPTS_DEFAULT, /* decoder_opts */                          \
+  }
 
 /*
  * options for the formatting phase (the syntax emitted / printed)
  */
 /* does not use numeric labels */
-#define IGA_FORMATTING_OPT_NUMERIC_LABELS   0x00000001u
+#define IGA_FORMATTING_OPT_NUMERIC_LABELS 0x00000001u
 /* enables certain syntax extensions */
-#define IGA_FORMATTING_OPT_SYNTAX_EXTS      0x00000002u
+#define IGA_FORMATTING_OPT_SYNTAX_EXTS 0x00000002u
 /* force floats to be emitted in hexadecimal */
 #define IGA_FORMATTING_OPT_PRINT_HEX_FLOATS 0x00000004u
 /* print the instruction PC's */
-#define IGA_FORMATTING_OPT_PRINT_PC         0x00000008u
+#define IGA_FORMATTING_OPT_PRINT_PC 0x00000008u
 /* print the instruction bits */
-#define IGA_FORMATTING_OPT_PRINT_BITS       0x00000010u
+#define IGA_FORMATTING_OPT_PRINT_BITS 0x00000010u
 /* print immediate instruction dependencies (mostly for IGA debugging) */
-#define IGA_FORMATTING_OPT_PRINT_DEPS       0x00000020u
+#define IGA_FORMATTING_OPT_PRINT_DEPS 0x00000020u
 /* print load/store pseduo instructions where possible */
-#define IGA_FORMATTING_OPT_PRINT_LDST       0x00000040u
+#define IGA_FORMATTING_OPT_PRINT_LDST 0x00000040u
 /* print bfn symbolic instructions */
-#define IGA_FORMATTING_OPT_PRINT_BFNEXPRS   0x00000080u
+#define IGA_FORMATTING_OPT_PRINT_BFNEXPRS 0x00000080u
 /* use ansi_span escapes when possible */
-#define IGA_FORMATTING_OPT_PRINT_ANSI       0x00000100u
+#define IGA_FORMATTING_OPT_PRINT_ANSI 0x00000100u
 /* emit JSON instead of asm */
-#define IGA_FORMATTING_OPT_PRINT_JSON       0x00000200u
+#define IGA_FORMATTING_OPT_PRINT_JSON 0x00000200u
 /* emit instruction definitions from a simple dataflow analysis */
-#define IGA_FORMATTING_OPT_PRINT_DEFS       0x00000400u
+#define IGA_FORMATTING_OPT_PRINT_DEFS 0x00000400u
+/* emit JSON old V1 version */
+#define IGA_FORMATTING_OPT_PRINT_JSON_V1 0x00000800u
 
 /* just the default formatting opts */
-#define IGA_FORMATTING_OPTS_DEFAULT \
-    (0u)
+#define IGA_FORMATTING_OPTS_DEFAULT (0u)
 /* a union of all formatter opts */
-#define IGA_FORMATTING_OPTS_ALL = \
-    (IGA_FORMATTING_OPT_NUMERIC_LABELS\
-    |IGA_FORMATTING_OPT_SYNTAX_EXTS\
-    |IGA_FORMATTING_OPT_PRINT_HEX_FLOATS\
-    |IGA_FORMATTING_OPT_PRINT_PC\
-    |IGA_FORMATTING_OPT_PRINT_BITS\
-    |IGA_FORMATTING_OPT_PRINT_DEPS\
-    |IGA_FORMATTING_OPT_PRINT_DEFS)
+#define IGA_FORMATTING_OPTS_ALL                                                \
+  = (IGA_FORMATTING_OPT_NUMERIC_LABELS | IGA_FORMATTING_OPT_SYNTAX_EXTS |      \
+     IGA_FORMATTING_OPT_PRINT_HEX_FLOATS | IGA_FORMATTING_OPT_PRINT_PC |       \
+     IGA_FORMATTING_OPT_PRINT_BITS | IGA_FORMATTING_OPT_PRINT_DEPS |           \
+     IGA_FORMATTING_OPT_PRINT_DEFS)
 
 /* uses the native decoder for decoding the kernel */
-#define IGA_DECODING_OPT_NATIVE   0x00000001u
+#define IGA_DECODING_OPT_NATIVE 0x00000001u
 /* just the default decoding opts */
-#define IGA_DECODING_OPTS_DEFAULT \
-    (0u)
+#define IGA_DECODING_OPTS_DEFAULT (0u)
+
 
 /*
  * Disassembles kernel bits into a string.
@@ -493,24 +479,15 @@ static_assert(sizeof(iga_disassemble_options_t) == 5*4,
  *  IGA_DECODE_ERROR    upon failure to decode error; specific error messages
  *                      may be retrieved via 'iga_context_get_errors'
  */
-IGA_API  iga_status_t  iga_context_disassemble(
-    iga_context_t ctx,
-    const iga_disassemble_options_t *opts,
-    const void *input,
-    uint32_t input_size,
-    const char *(*fmt_label_name)(int32_t, void *),
-    void *fmt_label_ctx,
-    char **kernel_text);
+IGA_API iga_status_t iga_context_disassemble(
+    iga_context_t ctx, const iga_disassemble_options_t *opts, const void *input,
+    uint32_t input_size, const char *(*fmt_label_name)(int32_t, void *),
+    void *fmt_label_ctx, char **kernel_text);
 /* deprecated API covers to iga_context_* */
-IGA_API  iga_status_t  iga_disassemble(
-    iga_context_t ctx,
-    const iga_disassemble_options_t *opts,
-    const void *input,
-    uint32_t input_size,
-    const char *(*fmt_label_name)(int32_t, void *),
-    void *fmt_label_ctx,
-    char **kernel_text);
-
+IGA_API iga_status_t iga_disassemble(
+    iga_context_t ctx, const iga_disassemble_options_t *opts, const void *input,
+    uint32_t input_size, const char *(*fmt_label_name)(int32_t, void *),
+    void *fmt_label_ctx, char **kernel_text);
 
 /*
  * Disassembles a single instruction.
@@ -545,22 +522,15 @@ IGA_API  iga_status_t  iga_disassemble(
  *  IGA_DECODE_ERROR    upon failure to decode error; specific error messages
  *                      may be retrieved via 'iga_context_get_errors'
  */
-IGA_API  iga_status_t  iga_context_disassemble_instruction(
-    iga_context_t ctx,
-    const iga_disassemble_options_t *dopts,
-    const void *input,
-    const char *(*fmt_label_name)(int32_t, void *),
-    void *fmt_label_ctx,
-    char **kernel_text);
+IGA_API iga_status_t iga_context_disassemble_instruction(
+    iga_context_t ctx, const iga_disassemble_options_t *dopts,
+    const void *input, const char *(*fmt_label_name)(int32_t, void *),
+    void *fmt_label_ctx, char **kernel_text);
 /* deprecated API covers to iga_contex* */
-IGA_API  iga_status_t  iga_disassemble_instruction(
-    iga_context_t ctx,
-    const iga_disassemble_options_t *dopts,
-    const void *input,
-    const char *(*fmt_label_name)(int32_t, void *),
-    void *fmt_label_ctx,
-    char **kernel_text);
-
+IGA_API iga_status_t iga_disassemble_instruction(
+    iga_context_t ctx, const iga_disassemble_options_t *dopts,
+    const void *input, const char *(*fmt_label_name)(int32_t, void *),
+    void *fmt_label_ctx, char **kernel_text);
 
 /*****************************************************************************/
 /*             Diagnostic Processing Functions                               */
@@ -574,19 +544,18 @@ IGA_API  iga_status_t  iga_disassemble_instruction(
  *          any conversion time down the road.
  */
 typedef struct {
-    /* lines start at 1; 0 means no location information */
-    uint32_t    line;
-    /* columns start at 1; 0 means no location information */
-    uint32_t    column;
-    /* the input offset */
-    uint32_t    offset;
-    /* the length of the diagnostic (in characters) */
-    uint32_t    extent;
-    /* the diagnostic message (NUL terminated) */
-    const char *message;
+  /* lines start at 1; 0 means no location information */
+  uint32_t line;
+  /* columns start at 1; 0 means no location information */
+  uint32_t column;
+  /* the input offset */
+  uint32_t offset;
+  /* the length of the diagnostic (in characters) */
+  uint32_t extent;
+  /* the diagnostic message (NUL terminated) */
+  const char *message;
 } iga_diagnostic_t;
 
-
 /*
  * Gets the errors from previous 'iga_context_assemble' or
  * 'iga_context_disassmble'.  It is an error to call this before one of these
@@ -613,30 +582,24 @@ typedef struct {
  *                      internal allocation fails, or the context is invalid
  *                      (after call to 'iga_context_release').
  */
-IGA_API iga_status_t iga_context_get_errors(
-    iga_context_t ctx,
-    const iga_diagnostic_t **ds,
-    uint32_t *ds_len);
-IGA_API iga_status_t iga_get_errors(
-    iga_context_t ctx,
-    const iga_diagnostic_t **ds,
-    uint32_t *ds_len);
-
+IGA_API iga_status_t iga_context_get_errors(iga_context_t ctx,
+                                            const iga_diagnostic_t **ds,
+                                            uint32_t *ds_len);
+IGA_API iga_status_t iga_get_errors(iga_context_t ctx,
+                                    const iga_diagnostic_t **ds,
+                                    uint32_t *ds_len);
 
 /*
  * Symmetric to 'iga_context_get_errors'.  Various API calls can still succeed
  * even if there are warnings; hence, robust tools should always check this
  * as well.
  */
-IGA_API iga_status_t iga_context_get_warnings(
-    iga_context_t ctx,
-    const iga_diagnostic_t **ds,
-    uint32_t *ds_len);
-IGA_API iga_status_t iga_get_warnings(
-    iga_context_t ctx,
-    const iga_diagnostic_t **ds,
-    uint32_t *ds_len);
-
+IGA_API iga_status_t iga_context_get_warnings(iga_context_t ctx,
+                                              const iga_diagnostic_t **ds,
+                                              uint32_t *ds_len);
+IGA_API iga_status_t iga_get_warnings(iga_context_t ctx,
+                                      const iga_diagnostic_t **ds,
+                                      uint32_t *ds_len);
 
 /*
  * Fetches a diagnostic's message.  The internal memory containing the message
@@ -647,10 +610,8 @@ IGA_API iga_status_t iga_get_warnings(
  *  IGA_SUCCESS           upon success
  *  IGA_INVALID_ARG       if an argument is NULL
  */
-IGA_API iga_status_t iga_diagnostic_get_message(
-    const iga_diagnostic_t *d,
-    const char **message);
-
+IGA_API iga_status_t iga_diagnostic_get_message(const iga_diagnostic_t *d,
+                                                const char **message);
 
 /*
  * Returns the starting file offset for the diagnostic in bytes (zero based).
@@ -661,23 +622,20 @@ IGA_API iga_status_t iga_diagnostic_get_message(
  *  IGA_INVALID_ARG       if an argument is NULL or the diagnostic is a binary
  *                        location
  */
-IGA_API iga_status_t iga_diagnostic_get_offset(
-    const iga_diagnostic_t *d,
-    uint32_t *offset);
-
+IGA_API iga_status_t iga_diagnostic_get_offset(const iga_diagnostic_t *d,
+                                               uint32_t *offset);
 
 /*
  * Diagnostics can refer to locations in binary files or text files.
  * The following enumeration indicates which.
  */
 typedef enum {
-    /* the location refers to a text location */
-    IGA_DIAGNOSTIC_TEXT,
-    /* the location refers to a binary location */
-    IGA_DIAGNOSTIC_BINARY,
+  /* the location refers to a text location */
+  IGA_DIAGNOSTIC_TEXT,
+  /* the location refers to a binary location */
+  IGA_DIAGNOSTIC_BINARY,
 } iga_diagnostic_type_t;
 
-
 /*
  * Fetches the type of diagnostic (binary or text).
  * For IGA_DIAGNOSTIC_TEXT iga_diagnostic_text_get_* should be used to access
@@ -690,10 +648,8 @@ typedef enum {
  *  IGA_INVALID_ARG       if an argument is NULL or the diagnostic is a binary
  *                        location
  */
-IGA_API iga_status_t iga_diagnostic_get_type(
-    const iga_diagnostic_t *d,
-    iga_diagnostic_type_t *dt);
-
+IGA_API iga_status_t iga_diagnostic_get_type(const iga_diagnostic_t *d,
+                                             iga_diagnostic_type_t *dt);
 
 /*
  * Fetches a diagnostic's line number (must be IGA_DIAGNOSTIC_TEXT).
@@ -704,19 +660,15 @@ IGA_API iga_status_t iga_diagnostic_get_type(
  *  IGA_INVALID_ARG       if an argument is NULL or the diagnostic is a binary
  *                        location
  */
-IGA_API iga_status_t iga_diagnostic_get_text_line(
-    const iga_diagnostic_t *d,
-    uint32_t *line);
-
+IGA_API iga_status_t iga_diagnostic_get_text_line(const iga_diagnostic_t *d,
+                                                  uint32_t *line);
 
 /*
  * Similar to 'iga_diagnostic_text_get_line', but returns the column.
  * Columns are counted from 1.
  */
-IGA_API iga_status_t iga_diagnostic_get_text_column(
-    const iga_diagnostic_t *d,
-    uint32_t *col);
-
+IGA_API iga_status_t iga_diagnostic_get_text_column(const iga_diagnostic_t *d,
+                                                    uint32_t *col);
 
 /*
  * Fetches a binary diagnostic's byte offset (must be IGA_DIAGNOSTIC_TEXT).
@@ -727,11 +679,8 @@ IGA_API iga_status_t iga_diagnostic_get_text_column(
  *  IGA_INVALID_ARG       if an argument is NULL or the diagnostic is a binary
  *                        location
  */
-IGA_API iga_status_t iga_diagnostic_get_text_extent(
-    const iga_diagnostic_t *d,
-    uint32_t *extent);
-
-
+IGA_API iga_status_t iga_diagnostic_get_text_extent(const iga_diagnostic_t *d,
+                                                    uint32_t *extent);
 
 /*****************************************************************************/
 /*             Operation Enumeration Functions                               */
@@ -740,7 +689,7 @@ IGA_API iga_status_t iga_diagnostic_get_text_extent(
  * An opaque type representing an operation (instruction) type.
  * Can be efficiently copied and passed by value.
  */
-typedef const struct opspec* iga_opspec_t;
+typedef const struct opspec *iga_opspec_t;
 
 /*
  * Enumerates the opcodes for a given platform.
@@ -766,11 +715,8 @@ typedef const struct opspec* iga_opspec_t;
  *  IGA_SUCCESS           upon success
  *  IGA_INVALID_ARG       if a gen argument invalid or ops_arr_len is null
  */
-IGA_API iga_status_t iga_opspec_enumerate(
-    iga_gen_t gen,
-    iga_opspec_t *ops_arr,
-    size_t *ops_arr_len);
-
+IGA_API iga_status_t iga_opspec_enumerate(iga_gen_t gen, iga_opspec_t *ops_arr,
+                                          size_t *ops_arr_len);
 
 /*
  * Looks up an opspec based on an iga::Op value.
@@ -784,11 +730,9 @@ IGA_API iga_status_t iga_opspec_enumerate(
  *  IGA_INVALID_ARG       if an argument is invalid
  *                        (invalid GEN or op or null os)
  */
-IGA_API iga_status_t iga_opspec_from_op(
-    iga_gen_t gen,
-    /* iga::Op */ uint32_t op_enum,
-    iga_opspec_t *os);
-
+IGA_API iga_status_t iga_opspec_from_op(iga_gen_t gen,
+                                        /* iga::Op */ uint32_t op_enum,
+                                        iga_opspec_t *os);
 
 /*
  * Lists the mnemonic name for a given op (e.g. "sends")
@@ -798,11 +742,8 @@ IGA_API iga_status_t iga_opspec_from_op(
  *  IGA_SUCCESS           upon success
  *  IGA_INVALID_ARG       if given an invalid argument
  */
-IGA_API iga_status_t iga_opspec_mnemonic(
-    const iga_opspec_t op,
-    char *mnemonic,
-    size_t *mnemonic_len);
-
+IGA_API iga_status_t iga_opspec_mnemonic(const iga_opspec_t op, char *mnemonic,
+                                         size_t *mnemonic_len);
 
 /*
  * Lists the op name for a given op (e.g. "Split Send Message")
@@ -812,11 +753,8 @@ IGA_API iga_status_t iga_opspec_mnemonic(
  *  IGA_SUCCESS           upon success
  *  IGA_INVALID_ARG       if a gen argument invalid
  */
-IGA_API iga_status_t iga_opspec_name(
-    const iga_opspec_t op,
-    char *name,
-    size_t *name_len);
-
+IGA_API iga_status_t iga_opspec_name(const iga_opspec_t op, char *name,
+                                     size_t *name_len);
 
 /*
  * Lists the op desription for a given op
@@ -826,11 +764,8 @@ IGA_API iga_status_t iga_opspec_name(
  *  IGA_SUCCESS           upon success
  *  IGA_INVALID_ARG       if a gen argument invalid
  */
-IGA_API iga_status_t iga_opspec_description(
-    iga_opspec_t op,
-    char *desc,
-    size_t *desc_len);
-
+IGA_API iga_status_t iga_opspec_description(iga_opspec_t op, char *desc,
+                                            size_t *desc_len);
 
 /*
  * Returns iga::Op for this operation as a uint32_t
@@ -839,10 +774,7 @@ IGA_API iga_status_t iga_opspec_description(
  *  IGA_SUCCESS           upon success
  *  IGA_INVALID_ARG       if an argument is NULL
  */
-IGA_API iga_status_t iga_opspec_op(
-    iga_opspec_t os,
-    uint32_t *op);
-
+IGA_API iga_status_t iga_opspec_op(iga_opspec_t os, uint32_t *op);
 
 /*
  * Returns iga::Op value opcode encoding.  This can vary for the same op
@@ -852,9 +784,7 @@ IGA_API iga_status_t iga_opspec_op(
  *  IGA_SUCCESS           upon success
  *  IGA_INVALID_ARG       if an argument is NULL
  */
-IGA_API iga_status_t iga_opspec_op_encoding(
-    iga_opspec_t op,
-    uint32_t *opcode);
+IGA_API iga_status_t iga_opspec_op_encoding(iga_opspec_t op, uint32_t *opcode);
 
 #ifdef __cplusplus
 }
